function csirdcf_tir_vot

addpath('E:\VOT\vot-toolkit-master\tracker\examples\matlab');

% *************************************************************
% VOT: Set random seed to a different value every time.
% *************************************************************
try
    % Simple check for Octave environment
    OCTAVE_VERSION;
    rand('seed', sum(clock));
    pkg load image;
catch
    RandStream.setGlobalStream(RandStream('mt19937ar', 'Seed', sum(clock)));
end

% **********************************
% VOT: Get initialization data
% **********************************
[handle, image, region] = vot('rectangle', 'rgbt');

%%%%% 
% Initialize the tracker
tracker_path = 'E:/VOT/vot-workspace/TIR2019/CISRDCF_TIR';
addpath(fullfile(tracker_path, 'external_libs/fhog'));
addpath(fullfile(tracker_path, '/external_libs/mtimesx'));
addpath(fullfile(tracker_path, '/external_libs/mexResize'));
addpath(fullfile(tracker_path, '/features'));

%%%%% Setting parameters for local use.
search_area_scale    = 4.25;     % 4.25
search_area_shape    = 'square'; % the shape of the training/detection window:
                                 % 'proportional', 'square' or 'fix_padding'
output_sigma_factor  = 1/10;
learning_rate_filt   = 0.025;
learning_rate_hist   = 0.125;
filter_max_area      = 38^2;     % 38
win_param            = 1;
nScales              = 3;
scale_step           = 1.03;
merge_coef_param     = 0.55;     % 0.55
reg_window_std       = 0.25;     % 0.25

% Coordinates interpolation
newton_iterations    = 5;  % 50
interpolate_response = 4;  % correlation score interpolation strategy:
                           % 0 - off, 1 - feature grid, 2 - pixel grid, 4 - Newton's method

% Global feature parameters
cell_size             = 4;
%cell_selection_thresh = 0.75^2; % Threshold for reducing the cell size in
                                % low-resolution cases
hog_params.nDim       = 31;
features = {
    ...struct('getFeature', @get_colorspace, 'fparams', grayscale_params), ...
    struct('getFeature', @get_fhog, 'fparams', hog_params),...
};
force_grayscale = 1;

% Visualization
visualize = false;

%%%%%
% Get a first frame
im = imread(image{1});
ir_im = imread(image{2});
if (force_grayscale && size(im, 3) == 3)
    im = rgb2gray(im);
end

% Check if it is color
if (size(im, 3) == 3)
    if (all(all(im(:, :, 1) == im(:, :, 2))))
        colorImage = false;
    else
        colorImage = true;
    end
else
    colorImage = false;
end

% 
rect = round(region);
%display(rect)

pos = floor([rect(2) + rect(4) / 2, rect(1) + rect(3) / 2]);
target_sz = floor([rect(4), rect(3)]);
init_target_sz = target_sz;

% Set the feature ratio to the feature-cell size
featureRatio = cell_size;
search_area = prod(init_target_sz / featureRatio * search_area_scale);

% when the number of cells are small, choose a smaller cell size
if (exist('cell_selection_thresh', 'var'))
    if (search_area < cell_selection_thresh * filter_max_area)
        cell_size = min(featureRatio, ...
                        max(1, ceil(sqrt(prod(init_target_sz * search_area_scale) / (cell_selection_thresh * filter_max_area)))));
        featureRatio = cell_size;
        search_area = prod(init_target_sz / featureRatio * search_area_scale);
    end
end

global_feat_params.cell_size = cell_size;
%global_feat_params.cell_selection_thresh = cell_selection_thresh;

if (search_area > filter_max_area)
    currentScaleFactor = sqrt(search_area / filter_max_area);
else
    currentScaleFactor = 1.0;
end

% target size at the initial scale
base_target_sz = target_sz / currentScaleFactor;

% window size, taking padding into account
switch (search_area_shape)
    case 'proportional'
        % proportional area, same aspect ratio as the target
        sz = floor(base_target_sz * search_area_scale);     
    case 'square'
        % square area, ignores the target aspect ratio
        sz = repmat(sqrt(prod(base_target_sz * search_area_scale)), 1, 2); 
    case 'fix_padding'
        % const padding
        sz = base_target_sz + ...
            sqrt(prod(base_target_sz * search_area_scale) + ...
            (base_target_sz(1) - base_target_sz(2)) / 4) - sum(base_target_sz) / 2; 
    otherwise
        error(['Unknown "params.search_area_shape". Must be '...
               '''proportional'', ''square'' or ''fix_padding''']);
end

% set the size to exactly match the cell size
sz = round(sz / featureRatio) * featureRatio;
use_sz = floor(sz/featureRatio);

% Construct the label function - correlation output, 2D gaussian function,
% with a peak located upon the target
output_sigma = sqrt(prod(floor(base_target_sz / featureRatio))) * output_sigma_factor;
rg           = circshift(-floor((use_sz(1) - 1) / 2):ceil((use_sz(1) - 1) / 2), ...
                         [0 -floor((use_sz(1) - 1) / 2)]);
cg           = circshift(-floor((use_sz(2) - 1) / 2):ceil((use_sz(2) - 1) / 2), ...
                         [0 -floor((use_sz(2) - 1) / 2)]);
% rg           = -floor((use_sz(1) - 1) / 2):ceil((use_sz(1) - 1) / 2);
% cg           = -floor((use_sz(2) - 1) / 2):ceil((use_sz(2) - 1) / 2);
[rs, cs]     = ndgrid(rg, cg);
y            = exp(-0.5 * (((rs .^ 2 + cs .^ 2) / output_sigma ^ 2)));
yf           = fft2(y); %   FFT of y.

if (interpolate_response == 1)
    interp_sz = use_sz * featureRatio;
else
    interp_sz = use_sz;
end

% Construct cosine window
%cos_window_l = single(hann(use_sz(1)) * hann(use_sz(2))');
cos_window = single(tukeywin(use_sz(1), win_param) * ...
                    tukeywin(use_sz(2), win_param)');

% Compute feature dimensionality
feature_dim = 0;
for n = 1:length(features)
    if (~isfield(features{n}.fparams, 'useForColor'))
        features{n}.fparams.useForColor = true;
    end
    
    if (~isfield(features{n}.fparams, 'useForGray'))
        features{n}.fparams.useForGray = true;
    end
    
    if ((features{n}.fparams.useForColor && colorImage) || ...
        (features{n}.fparams.useForGray && ~colorImage))
        feature_dim = feature_dim + features{n}.fparams.nDim;
    end
end

if (size(im, 3) > 1 && colorImage == false)
    im = im(:, :, 1);
end

if (nScales > 0)
    scale_exp = (-floor((nScales-1)/2):ceil((nScales-1)/2));
    scaleFactors = scale_step .^ scale_exp;
    min_scale_factor = scale_step ^ ceil(log(max(5 ./ sz)) / log(scale_step));
    max_scale_factor = scale_step ^ floor(log(min([size(im,1) size(im,2)] ./ base_target_sz)) / log(scale_step));
    
    scaleWeights = exp(-0.5 * scale_exp .^ 2 / 16);
end

if (interpolate_response >= 3)
    % Pre-computes the grid that is used for socre optimization
    ky = circshift(-floor((use_sz(1) - 1)/2) : ceil((use_sz(1) - 1)/2), ...
                   [1, -floor((use_sz(1) - 1)/2)]);
    kx = circshift(-floor((use_sz(2) - 1)/2) : ceil((use_sz(2) - 1)/2), ...
                   [1, -floor((use_sz(2) - 1)/2)])';
end

% allocate memory for multi-scale tracking
%multires_pixel_template = zeros(sz(1), sz(2), size(im,3), nScales, 'uint8');
%small_filter_sz = floor(base_target_sz / featureRatio);

frame = 1;
while true

    if (frame > 1)
        % **********************************
        % VOT: Get next frame
        % **********************************
    
        [handle, image] = handle.frame(handle);

        if isempty(image)
            break;
        end;

        % Perform a tracking step, obtain new region
        im = imread(image{1});
        ir_im = imread(image{2});
        if (force_grayscale && size(im, 3) == 3)
            im = rgb2gray(im);
        end

        if (size(im, 3) > 1 && colorImage == false)
            im = im(:, :, 1);
        end

        % do not estimate translation and scaling on the first frame,
        % since we just want to initialize the tracker there
        for scale_ind = 1:nScales
            pixels = get_pixels(im, pos, round(sz*currentScaleFactor*scaleFactors(scale_ind)), sz);
            ir_pixels = get_pixels(ir_im, pos, round(sz*currentScaleFactor*scaleFactors(scale_ind)), sz);
            %multires_pixel_template(:, :, :, scale_ind) = pixels;
            
            feat = get_features(pixels, features, global_feat_params);
            ir_feat = get_features(pixels, features, global_feat_params);
            feat_sz = [size(cos_window, 1), size(cos_window, 2)];
            f_hist = get_hist_features(pixels, nhist, feat_sz);
            ir_f_hist = get_hist_features(ir_pixels, ir_nhist, feat_sz);
            feat = cat(3, feat, ir_feat, f_hist, ir_f_hist);
            
            f(:, :, :, scale_ind) = feat;
        end
        xtf = fft2(bsxfun(@times, f, cos_window));
        
        corr = bsxfun(@times, gm_f, xtf);
        corr1 = sum(corr(:, :,   1:62, :), 3);
        corr2 = sum(corr(:, :, 63:end, :), 3);
        
        responsef1 = permute(corr1, [1 2 4 3]);
        responsef2 = permute(corr2, [1 2 4 3]);
        
        %responsef = permute(sum(bsxfun(@times, gm_f, xtf), 3), [1 2 4 3]);
        for scale_ind = 1:nScales
            responsef1(:, :, scale_ind) = responsef1(:, :, scale_ind) * ...
                                             scaleWeights(scale_ind);
        end
        
        % if we undersampled features, we want to interpolate the
        % response so it has the same size as the image patch
        if (interpolate_response == 2)
            % use dynamic interp size
            interp_sz = floor(size(y) * featureRatio * currentScaleFactor);
        end
        responsef1_padded = resizeDFT2(responsef1, interp_sz);
        responsef2_padded = resizeDFT2(responsef2, interp_sz);
        
        % response in the spatial domain
        response1 = ifft2(responsef1_padded, 'symmetric');
        response2 = ifft2(responsef2_padded, 'symmetric');
        
        [r1_max, ind1] = max(response1(:));
        [r2_max, ind2] = max(response2(:));
        [i_max, j_max, s_max] = ind2sub(size(response1), ind1);
        %[i2_max, j2_max, s2_max] = ind2sub(size(response2), ind2);
        
        merge_coef = r1_max / r2_max * merge_coef_param;
        response = response1 + merge_coef * response2;
        responsef_padded = responsef1_padded + merge_coef * responsef2_padded;
        
        % find maximum peak
        if (interpolate_response == 3)
            error('Invalid parameter value for interpolate_response');
        elseif (interpolate_response == 4)
            sind = s_max;
            [disp_row, disp_col, ~] = resp_newton(response(:, :, sind), responsef_padded(:, :, sind), newton_iterations, ky, kx, use_sz);
            %[disp_row, disp_col, sind] = resp_newton(response, responsef_padded, newton_iterations, ky, kx, use_sz);
        else
            [row, col, sind] = ind2sub(size(response), find(response == max(response(:)), 1));
            disp_row = mod(row - 1 + floor((interp_sz(1)-1)/2), interp_sz(1)) - floor((interp_sz(1)-1)/2);
            disp_col = mod(col - 1 + floor((interp_sz(2)-1)/2), interp_sz(2)) - floor((interp_sz(2)-1)/2);
        end
        
        scale = scaleFactors(sind);
        % calculate translation
        switch (interpolate_response)
            case 0
                translation_vec = round([disp_row, disp_col] * featureRatio * currentScaleFactor * scale);
            case 1
                translation_vec = round([disp_row, disp_col] * currentScaleFactor * scale);
            case 2
                translation_vec = round([disp_row, disp_col] * scale);
            case 3
                translation_vec = round([disp_row, disp_col] * featureRatio * currentScaleFactor * scale);
            case 4
                translation_vec = round([disp_row, disp_col] * featureRatio * currentScaleFactor * scale);
        end
        
        % set the scale
        currentScaleFactor = currentScaleFactor * scale;
        % adjust to make sure we are not to large or to small
        if (currentScaleFactor < min_scale_factor)
            currentScaleFactor = min_scale_factor;
        elseif (currentScaleFactor > max_scale_factor)
            currentScaleFactor = max_scale_factor;
        end
        
        % update position
        pos = pos + translation_vec;
    end
    
    % extract training sample image region
    pixels = get_pixels(im, pos, round(sz * currentScaleFactor), sz);
    ir_pixels = get_pixels(ir_im, pos, round(sz * currentScaleFactor), sz);
    
    % extract features and do windowing
    f = get_features(pixels, features, global_feat_params);
    ir_f = get_features(ir_pixels, features, global_feat_params);
    if (frame == 1)
        nhist = get_hist(pixels, round(target_sz / currentScaleFactor));
        ir_nhist = get_hist(ir_pixels, round(target_sz / currentScaleFactor));
    end
    feat_sz = [size(cos_window, 1), size(cos_window, 2)];
    f_hist = get_hist_features(pixels, nhist, feat_sz);
    ir_f_hist = get_hist_features(ir_pixels, ir_nhist, feat_sz);
    f = cat(3, f, ir_f, f_hist, ir_f_hist);
    
    F = fft2(bsxfun(@times, f, cos_window));
     
    %%%%%%%%%%
    % ADMM
    if (frame == 1)
        R = yf;
        mu = 1;
        maxU = 1000;
        numIt = 2;
        beta = 30;  % 50
        G = zeros(size(F));
        L = zeros(size(F));
        
        p = 100;
        m = zeros(use_sz, 'single');
        tf_sz = round(base_target_sz / featureRatio);
        tf_sz = min(tf_sz, use_sz);
        m((1:tf_sz(1)) + floor((use_sz(1) - tf_sz(1)) / 2), ...
          (1:tf_sz(2)) + floor((use_sz(2) - tf_sz(2)) / 2)) = p;
        m = imfilter(m, fspecial('gaussian', [7, 7], reg_window_std));
        w = p - m;
        
        w2 = 2 * w .^ 2;
    end
    
    u = mu;
    %L = zeros(size(F));
    %G = zeros(size(F));
    FR = bsxfun(@times, conj(F), R);
    FF = conj(F) .* F;
    for i = 1:numIt
        H = (FR + (u / 2) .* (G - L)) ./ (FF + (u / 2));
        
        w = (u / 2) ./ (w2 + (u / 2));
        G = fft2(bsxfun(@times, real(ifft2(H + L)), w));
        
        L = L + (H - G);

        if (u ~= maxU)
            u_ = u;
            u = min(beta * u, maxU);
            L = L * (u_ / u);
        end
    end
    
    %%%%% update
    if (frame == 1)
        gm_f = G;
    else
        gm_f = ((1 - learning_rate_filt) * gm_f) + (learning_rate_filt * G);
        nhist = (1 - learning_rate_hist) * nhist + ...
              learning_rate_hist * get_hist(pixels, round(target_sz / currentScaleFactor));
        ir_nhist = (1 - learning_rate_hist) * ir_nhist + ...
              learning_rate_hist * get_hist(ir_pixels, round(target_sz / currentScaleFactor));
    end
    
    target_sz = floor(base_target_sz * currentScaleFactor);
    
    % Pposition
    region = double([pos([2,1]) - target_sz([2,1])/2, target_sz([2,1])]);
    frame = frame + 1;
    
%     if (visualize)
%         if (~exist('h_img', 'var'))
%             h_img = imshow(im);
%             h_rect = rectangle('Position', region);
%         else
%             set(h_img, 'CData', im);
%             set(h_rect, 'Position', region);
%         end        
%     end
    
    % **********************************
    % VOT: Report position for frame
    % **********************************
    handle = handle.report(handle, region);

end;

% **********************************
% VOT: Output the results
% **********************************
handle.quit(handle);

end
